###################################################################
#
# Code to produce Figure 1 in: Burgess, Snyder, Rountree (xxxx)
# Collective dispersal leads to variance in fitness and maintains
# offspring size variation within marine populations. The American
# Naturalist #: ##-##
#
# Based on in-house file plotMeanVarLottery.r
#
####################################################################

N = 100  			# number of sites
delta = 0.8  		# probability of adult death
s0 = 0.0103			# larval size at which devlopment takes t0 days
smin = 0.001		# minimum viable larval size
t0 = 14         	# pelagic larval duration for initial size s0
mLo = 0.05          # larval mortality rate
mHi = 0.5
tau = 15			# "Eddy time scale" --- 15 days acc. to Siegel et al., 2008
h = 0.5				# probability of hitting the coastline with viable habitat
f0 = 100*s0			# reproductive investment
spatialDomainToEddySize = 10     # spatial domain / eddy size

n1 = 1
n2 = N - 1

# PLD as a function of s
t = function (s) {
  (s0/s - 1) + t0
}

# Per capita number of larvae post-mortality
fec = function (s, mort, f0) {
  f0/s * exp(-mort*t(s))
}

# Number of water parcels among which larvae are divided
M = function (tee) {
  ceiling ((tee / tau) * spatialDomainToEddySize)
}

# Expected number of settlers
Ey1 = function (s) fec(s,mort,f0)*h*n1
Ey2 = function (s) fec(s,mort,f0)*h*n2

# Variance in number of settlers
Vary1 = function (s, tee) fec(s,mort,f0)^2/M(tee) * h*(1 - h) * n1^2
Vary2 = function (s, tee) fec(s,mort,f0)^2/M(tee) * h*(1 - h) * n2^2

numreps = 10^5 # Making this large will cause very long run times, especially with small N
simMeanVar1 = function (s1, s2, tee1, tee2) {
  w1 = numeric (numreps)
  for (i in 1:numreps) {
    ## y1 = number of type 1 settlers that reach habitat
    y1 = fec(s1,mort,f0)*n1/M(tee1) * rbinom (1, prob=h, size=M(tee1))
    y2 = fec(s2,mort,f0)*n2/M(tee2) * rbinom (1, prob=h, size=M(tee2))
    ## w1 = number of type 1 recruits + type 1 survivors (= fitness)
    w1[i] = y1/(y1 + y2) * delta*N + (1 - delta)*n1
  }
  meanw = mean(w1); varw = var(w1)
  return (list(mean=meanw, var=varw))
}

######################################


# Define types
ns1 = 30
ntee1 = 3

lowerSpawningTime = 10 #100
upperSpawningTime = 90 #300 	# ceiling(M(300)) = 100 packets

s1Series = seq(smin, s0, length=ns1)
tee1Series = floor(seq(lowerSpawningTime, upperSpawningTime, length.out=ntee1))
tee2 = 0.5*(lowerSpawningTime + upperSpawningTime)

mort = mLo
tmp_fec_s <- fec(s1Series,mort,f0)*h
s2Lo = s1Series[which(tmp_fec_s==max(tmp_fec_s))]

meanFitnessmLo = matrix (NA, ns1, ntee1)
varFitnessmLo = matrix (NA, ns1, ntee1)

for (i in 1:ns1) {
  s1 = s1Series[i]
  for (j in 1:ntee1) {
    tee1 = tee1Series[j]
    out = simMeanVar1 (s1, s2Lo, tee1, tee2)
    meanFitnessmLo[i,j] = out$mean
    varFitnessmLo[i,j] = out$var
  }
}

mort = mHi
tmp_fec_s <- fec(s1Series,mort,f0)*h
s2Hi = s1Series[which(tmp_fec_s==max(tmp_fec_s))]
meanFitnessmHi = matrix (NA, ns1, ntee1)
varFitnessmHi = matrix (NA, ns1, ntee1)

for (i in 1:ns1) {
  s1 = s1Series[i]
  for (j in 1:ntee1) {
    tee1 = tee1Series[j]
    out = simMeanVar1 (s1, s2Hi, tee1, tee2)
    meanFitnessmHi[i,j] = out$mean
    varFitnessmHi[i,j] = out$var
  }
}


# Make plot
label.cex=1.4
biglabel.cex=1.9

#quartz(width=9,height=7)
dev.new(width=9,height=7)
perspMar = c(3,3,0,0)
regMar = c(4,3,1.5,2)

par (mfrow=c(2, 2), cex=1, cex.axis=0.8, cex.lab=label.cex, oma=c(1,3,4,0),
     mgp=c(3,1,0))

add_panel_label <- function(ltype="a") {
    text <- paste(LETTERS[letters==ltype], ")", sep="")
    mtext(text=text, side=3, adj=0)
}


#a
par(mar=regMar)
plot(s1Series,meanFitnessmLo[,2],type="l",xlab="",ylab="",las=1,lwd=2,ylim=c(0,max(meanFitnessmLo)))
add_panel_label("a")
mtext(expression(paste("Offspring size, ", s[1], " (mm"^3,")")),side=1,line=3,
      cex=label.cex)
mtext("Expected fitness \n(number of recruits)",side=2,line=2.5,
      cex=label.cex)

abline(v=s2Lo,lty=2)

#b
par(mar=regMar)
plot(s1Series,meanFitnessmHi[,2],type="l",xlab="", ylab="",las=1,lwd=2,ylim=c(0,max(meanFitnessmHi)))
add_panel_label("b")
mtext(expression(paste("Offspring size, ", s[1], " (mm"^3,")")),side=1,line=3,
      cex=label.cex)
mtext("Expected fitness \n(number of recruits)",side=2,line=2,
      cex=label.cex)

abline(v=s2Hi,lty=2)

#c
par(mar=regMar)
plot(s1Series,varFitnessmLo[,2],
     type="l",xlab="",ylab="",las=1,lwd=2,ylim=c(0,max(varFitnessmLo)))
lines(s1Series,varFitnessmLo[,1],lwd=2,lty=2)
lines(s1Series,varFitnessmLo[,3],lwd=2,lty=3)
add_panel_label("c")
mtext(expression(paste("Offspring size, ", s[1], " (mm"^3,")")),side=1,line=3,
      cex=label.cex)
mtext("Fitness variance",side=2,line=3,
      cex=label.cex)     
tmp <- c("","T2 =","")
legend(x=smin*2,y=max(varFitnessmLo),	
	legend=eval(paste("T1=",tmp,tee1Series,"; M(T)=",ceiling(M(tee1Series)))),
	lty=c(2,1,3),lwd=2,
	bty="n")

abline(v=s2Lo,lty=2)


#d
par(mar=regMar)
plot(s1Series,varFitnessmHi[,2],
     type="l",xlab="",ylab="",las=1,lwd=2,lty=1,ylim=c(0,max(varFitnessmHi)))
lines(s1Series,varFitnessmHi[,1],lwd=2,lty=2)
lines(s1Series,varFitnessmHi[,3],lwd=2,lty=3)
add_panel_label("d")
mtext(expression(paste("Offspring size, ", s[1], " (mm"^3,")")),side=1,line=3,
      cex=label.cex)
mtext("Fitness variance",side=2,line=3,
      cex=label.cex)

abline(v=s2Hi,lty=2)


mtext("Low offspring mortality",side=3, cex=biglabel.cex,
      line=2, outer=TRUE, adj=0.1)
mtext("(m = 0.05)", side=3, cex=biglabel.cex, line=0.2, outer=TRUE, adj = 0.25)
mtext("High offspring mortality",side=3,
      cex=biglabel.cex, line=2, outer=TRUE, adj=0.93)
mtext("(m = 0.5)", side=3, cex=biglabel.cex, line=0.2, outer=TRUE, adj = 0.83)


# setwd("")
# install.packages("extrafont")
library(extrafont)
loadfonts(device="postscript")
dev.copy2pdf (file="Figure_1.pdf", family="ArialMT",useDingbats=FALSE)
# Embed text for Am Nat
Sys.setenv(R_GSCMD="/usr/local/bin/gs")
embed_fonts ("Figure_1.pdf",outfile='Figure_1.pdf')

# Save data in the workspace
save.image(file="Figure_1.RData")



