#################################################################
#
# Code to produce Figure 2 in: Burgess, Snyder, Rountree (xxxx)
# Collective dispersal leads to variance in fitness and maintains
# offspring size variation within marine populations. The American
# Naturalist #: ##-##
#
# Based on in-house script called pipConstT.r
#
################################################################

ns = 30 			# number of sizes s to use
ntee = 30 			# number of spawning durations T to use

N = 100  			# number of sites
delta = 0.8  		# probability of adult death
s0 = 0.0103			# larval size at which devlopment takes t0 days
smin = 0.001		# minimum viable larval size
t0 = 14         	# pelagic larval duration for initial size s0
mLo = 0.05          # larval mortality rate
mHi = 0.5
tau = 15			# "Eddy time scale" --- 15 days acc. to Siegel et al., 2008
h = 0.5				# probability of hitting the coastline with viable habitat
f0 = 100*s0			# reproductive investment
spatialDomainToEddySize = 10     # spatial domain / eddy size

n1 = 1
n2 = N - 1

# PLD as a function of s
t = function (s) {
  (s0/s - 1) + t0
}

# Per capita number of larvae post-mortality
fec = function (s, mort, f0) {
  f0/s * exp(-mort*t(s))
}

# Number of water parcels among which larvae are divided
M = function (tee) {
  ceiling ((tee / tau) * spatialDomainToEddySize)
}

n1Series = 0:N
sSeries = seq(smin, s0, length=ns)	# vector of larval sizes
teeSeries = seq(9, 180, length=ntee)    # vector of spawning durations

getq1 = function (s1, s2, tee1, tee2) {
  m1 = M (tee1); m2 = M (tee2)
  b1 = fec(s1, mort, f0); b2 = fec(s2, mort, f0)
  P = matrix (0, N+1, N+1)
  
  ## Set up transition matrix #############
  ## loop over current value of p
  for (n1 in n1Series) {
    p = n1/N
    index = n1 + 1
    ## What are the possible transitions we could make from current
    ## value of p to pNew?

    ## loop over number of successful type 1 packets
    for (k1 in 0:m1) {
      ## loop over number of successful type 2 packets
      for (k2 in 0:m2) {
        ## Calculate the updated value of p.
        numType1Larvae = b1*k1*p/m1
        numType2Larvae = b2*k2*(1 - p)/m2
        pNew = delta*numType1Larvae / (numType1Larvae  + numType2Larvae) +
          (1 - delta)*p
        if ((numType1Larvae == 0) & (numType2Larvae == 0)) pNew = p
        
        ## Round the updated n to the nearest integer since the
        ## proportion of space won is deterministic.
        n1New = round(pNew*N)
        pNew = n1New/N
        newIndex = n1New + 1
        
        ## Calculate the probability of getting k1 packets for type
        ## 1 and k2 packets for type 2.
        prob = dbinom (k1, m1, h) * dbinom (k2, m2, h)

        ## Fill in the relevant column of the transition matrix. 
        P[newIndex, index] = P[newIndex, index] + prob
      }
    }
  }

  ## a1 is the probability of fixating at p = 1 in the next time step.
  a1 = P[N+1, 2:N]

  ## Calculate the fundamental matrix of the transient states
  fundN = solve (diag(N-1) - P[2:N, 2:N])

  q1 = (a1 %*% fundN)[1]  # Start with 1 individ. of type 1
  return (q1)
}

q1s = as.list (1:2); q1tee = as.list (1:2)
mortlist = c(mLo, mHi)
tee1 = 50; tee2 = tee1

for (mm in 1:2) {
  mort = mortlist[mm]
  q1s[[mm]] = matrix (NA, ns, ns)

  for (i in 1:ns) { # resident
    for (j in 1:ns) { # invader
      s1 = sSeries[j]; s2 = sSeries[i]
      q1s[[mm]][i,j] = getq1(s1, s2, tee1, tee2)
    }
  }
}

s1 = 0.006; s2 = s1
for (mm in 1:2) {
  mort = mortlist[mm]
  q1tee[[mm]] = matrix (NA, ntee, ntee)

  for (i in 1:ntee) { # resident
    for (j in 1:ntee) { # invader
      tee1 = teeSeries[j]; tee2 = teeSeries[i]
      q1tee[[mm]][i,j] = getq1(s1, s2, tee1, tee2)
    }
  }
}


# Make plot
temp1 = rbind (matrix(1,5,5), matrix(2,5,5))
temp2 = rbind (matrix(3,5,5), matrix(4,5,5))
layout (mat=cbind(temp1, temp2, rep(5,10)))
myGrey = colorRampPalette (c("white", "black"))
lab.cex=1.5; main.cex=1.5

add_panel_label <- function(ltype="a") {
    text <- paste(LETTERS[letters==ltype], ")", sep="")
    mtext(text=text, side=3, adj=0)
}

panelLabel = c("a", "c", "b", "d")
for (mm in 1:2) {
  mort = mortlist[mm]
  title = paste ("m = ", mort, sep="")
  par (mar=c(5,4.5,3,1))
  image (sSeries, sSeries, q1s[[mm]], col=myGrey(100), 
         xlab="Resident offspring size", ylab="",
         cex.lab=lab.cex, cex.main=main.cex, main="",las=1)
  mtext("Invader offspring size",side=2,line=3.4)
  mtext(title,side=3)
  
  tmp_fec_s <- fec(sSeries,mort,f0)*h
  maxExpFitness = sSeries[which(tmp_fec_s==max(tmp_fec_s))]
  points (maxExpFitness, maxExpFitness, col="white", pch=8, cex=1.5)
  add_panel_label (panelLabel[(mm-1)*2 + 1])
  
  image (teeSeries, teeSeries, q1tee[[mm]], col=myGrey(100),
         xlab="Resident spawning duration", ylab="",
         cex.lab=lab.cex,las=1)
  mtext("Invader spawning duration",side=2,line=3)

  points (teeSeries[ntee], teeSeries[ntee], col="white", pch=17, cex=1.5)
  add_panel_label (panelLabel[(mm-1)*2 + 2])
}

l.y=seq(0,1,length=100); par(mar=c(6,2,3,1))
image(list(x=1:2,y=l.y,z=rbind(l.y,l.y)),col=myGrey(100),bty="o",xaxt="n",yaxt="n");
axis(side=2,cex.axis=1.5,at=pretty(seq(0,1,length=10)),las=1);


# setwd("")
# install.packages("extrafont")
library(extrafont)
loadfonts(device="postscript")
dev.copy2pdf (file="Figure_2.pdf", family="ArialMT",useDingbats=FALSE)
# Embed text for Am Nat
Sys.setenv(R_GSCMD="/usr/local/bin/gs")
embed_fonts ("Figure_2.pdf",outfile='Figure_2.pdf')

# Save data in the workspace
save.image(file="Figure_2.RData")



