###################################################################
#
# Code to produce Figure 4 in: Burgess, Snyder, Rountree (xxxx)
# Collective dispersal leads to variance in fitness and maintains
# offspring size variation within marine populations. The American
# Naturalist #: ##-##
#
# Based on in-house script called twoTraitCoexGreyscale2.r
#
##################################################################

ns = 30	 			# number of sizes s to use
N = 100  			# number of sites
delta = 0.8  		# probability of adult death
s0 = 0.0103			# larval size at which devlopment takes t0 days
smin = 0.001		# minimum viable larval size
t0 = 14         	# pelagic larval duration for initial size s0
mLo = 0.05          # larval mortality rate
mHi = 0.5
mort = mHi
tau = 15			# "Eddy time scale" --- 15 days acc. to Siegel et al., 2008
h = 0.5				# probability of hitting the coastline with viable habitat
f0 = 100*s0			# reproductive investment
spatialDomainToEddySize = 10     # spatial domain / eddy size

n1 = 1
n2 = N - 1

# PLD as a function of s
t = function (s) {
  (s0/s - 1) + t0
}

# Per capita number of larvae post-mortality
fec = function (s, mort, f0) {
  f0/s * exp(-mort*t(s))
}

# Number of water parcels among which larvae are divided
M = function (tee) {
  ceiling ((tee / tau) * spatialDomainToEddySize)
}

n1Series = 0:N
sSeries = seq(smin, s0, length=ns)	# vector of larval sizes
teeSeries = c(21, 51, 102)   # vector of spawning durations
ntee = length (teeSeries)

getMeanTimeToFixation = function (s1, s2, tee1, tee2) {
  b1 = fec(s1, mort, f0); b2 = fec(s2, mort, f0)
  m1 = M (tee1); m2 = M (tee2)
  P = matrix (0, N+1, N+1)
  
  ## Set up transition matrix #############
  ## loop over current value of p
  for (n1 in n1Series) {
    p = n1/N
    index = n1 + 1
    ## What are the possible transitions we could make from current
    ## value of p to pNew?

    ## loop over number of successful type 1 packets
    for (k1 in 0:m1) {
      ## loop over number of successful type 2 packets
      for (k2 in 0:m2) {
        ## Calculate the updated value of p.
        numType1Larvae = b1*k1*p/m1
        numType2Larvae = b2*k2*(1 - p)/m2
        pNew = delta*numType1Larvae / (numType1Larvae  + numType2Larvae) +
          (1 - delta)*p
        if ((numType1Larvae == 0) & (numType2Larvae == 0)) pNew = p
        
        ## Round the updated n to the nearest integer since the
        ## proportion of space won is deterministic.
        n1New = round(pNew*N)
        pNew = n1New/N
        newIndex = n1New + 1
        
        ## Calculate the probability of getting k1 packets for type
        ## 1 and k2 packets for type 2.
        prob = dbinom (k1, m1, h) * dbinom (k2, m2, h)

        ## Fill in the relevant column of the transition matrix. 
        P[newIndex, index] = P[newIndex, index] + prob
      }
    }
  }

  ## Calculate the fundamental matrix of the transient states
  fundN = solve (diag(N-1) - P[2:N, 2:N])

  ## Mean time with both types if type 1 is invading
  tmp = apply (fundN, 2, sum)
  lifetimeInv1 = tmp[1]
  lifetimeInv2 = tmp[N-1]
  return (list(lifetimeInv1=lifetimeInv1, lifetimeInv2=lifetimeInv2))
}

lifetime1 = array (NA, dim=c(ntee, ntee, ns, ns))
lifetime2 = array (NA, dim=c(ntee, ntee, ns, ns))
both = array (NA, dim=c(ntee, ntee, ns, ns))

for (i in 1:ntee) {
  cat ("i = ", i, "\n")
  tee2 = teeSeries[i]
  for (j in 1:ntee) {
    cat ("j = ", j, "\n")
    tee1 = teeSeries[j] 
    for (k in 1:ns) {
      s2 = sSeries[k]
      for (m in 1:ns) {
        s1 = sSeries[m]
        foo = getMeanTimeToFixation (s1, s2, tee1, tee2)
        lifetime1[i,j,k,m] = foo$lifetimeInv1
        lifetime2[i,j,k,m] = foo$lifetimeInv2
        both[i,j,k,m] = log(min(c(lifetime1[i,j,k,m], lifetime2[i,j,k,m])))
      }
    }
  }
}


#define greyscale
myGrey = colorRampPalette (c("white", "black"))

#Set min and max z values
zmin = min(both)
zmax = max(both)

graphics.off()

## Note: you must plot the color strip first, otherwise it comes out
## square instead of as a strip!  Why?  Do not ask such questions!!
l.y=seq(zmin, zmax, length=100);
par(mar=c(0,0,0,0), fig=c(0.94, 0.99, 0.1, 0.9))
image(list(x=1:2,y=l.y,z=rbind(l.y,l.y)),col=myGrey(100), bty="o",xaxt="n",yaxt="n");
tickLabels = c(1, 10, 10^2, 10^3, 10^4, 10^5, 10^6, 10^7)
tickLabels = tickLabels[tickLabels < exp(zmax)]
tickLocations = log(tickLabels)
axis(side=2,cex.axis=1.1,at=tickLocations, label=tickLabels)
mtext("Coexistence time (years)",side=2,line=3,cex=1.4)

par (pty="s", mar=c(5.1, 4.1, 4.1, 4.1), fig=c(0,1,0,1), new=T)

## plot invisible graph so that we can lay down outer axes
title = paste ("m = ", mort, sep="")
faketee = 0:(ntee+1)
plot (faketee, faketee, type="n", xlab="Spawning duration of type 2",
      ylab="Spawning duration of type 1", xaxt="n", yaxt="n", asp=1,
      cex.lab=1.4, 
      xlim=c(0.2, ntee+0.8), ylim=c(0.2, ntee+0.8),bty="n")
axis (side=1, at=1:ntee, labels=teeSeries,line=0.5)
axis (side=2, at=1:ntee, labels=teeSeries,line=0.5)
mtext(title,side=3,line=-2,cex=1.4)

## find the size that maximizes expected fecundity
tmp_fec_s <- fec(sSeries,mort,f0)*h
sexp = sSeries[which(tmp_fec_s==max(tmp_fec_s))]

## convert from user coordinates to normalized device coordinates
xlow = grconvertX (1:ntee - 0.45, from="user", to="ndc")
xhigh = grconvertX (1:ntee + 0.45, from="user", to="ndc")
ylow = xlow; yhigh = xhigh

## plot the subplots
for (i in 1:ntee) {
  for (j in 1:ntee) {
    par (fig=c(xlow[i], xhigh[i], ylow[j], yhigh[j]), new=T, mar=c(0,0,0,0))
    ## Only plot if there are some non-zero parts
    if (mean (both[i,j,,]) != 0) {
      if ((i == 1) & (j == 1)) { #axis labels on bottom left plot
        image (sSeries, sSeries, both[i,j,,], xlab="", ylab="", bty="l",
               col=myGrey(100), zlim=c(zmin, zmax))
        mtext ("s2", 1, line=1.8)
        mtext ("s1", 2, line=1.8)
      } else {
        image (sSeries, sSeries, both[i,j,,], xlab="", ylab="",
               col=myGrey(100), xaxt="n", yaxt="n", zlim=c(zmin, zmax))
      }
      points (sexp, sexp, col="white", pch=8)
    } else {
      plot (sSeries, sSeries, type="n", xaxt="n", yaxt="n")
    }
  }
}

# setwd("")
# install.packages("extrafont")
library(extrafont)
loadfonts(device="postscript")
dev.copy2pdf (file="Figure_4.pdf", family="ArialMT",useDingbats=FALSE)
# Embed text for Am Nat
Sys.setenv(R_GSCMD="/usr/local/bin/gs")
embed_fonts ("Figure_4.pdf",outfile='Figure_4.pdf')

# Save data in the workspace
save.image(file="Figure_4.RData")
