###################################################################
#
# Code to produce Figure 5 in: Burgess, Snyder, Rountree (xxxx)
# Collective dispersal leads to variance in fitness and maintains
# offspring size variation within marine populations. The American
# Naturalist #: ##-##
#
# IMPORTANT: This code only generates the figure.  To get the required
# data, run nonspatialPipAndFixation.r and
# spatialPipAndFixation2Parallel.r and save the output with
# appropriate file names.
#
# Based on in-house script called makePip1PatchTwoPatch.r
#
##################################################################


N = 100  			# number of sites
delta = 0.8  		# probability of adult death
s0 = 0.0103			# larval size at which devlopment takes t0 days
smin = 0.001		# minimum viable larval size
t0 = 14         	# pelagic larval duration for initial size s0
mLo = 0.05          # larval mortality rate
mHi = 0.5
tau = 15			# "Eddy time scale" --- 15 days acc. to Siegel et al., 2008
h = 0.5				# probability of hitting the coastline with viable habitat
f0 = 100*s0			# reproductive investment
spatialDomainToEddySize = 10     # spatial domain / eddy size

## PLD as a function of s
t = function (s) {
  (s0/s - 1) + t0
}

## Per capita number of larvae post-mortality
fec = function (s, mort, f0) {
  f0/s * exp(-mort*t(s))
}

# Number of water parcels among which larvae are divided
M = function (tee) {
  ceiling ((tee / tau) * spatialDomainToEddySize)
}

n1Series = 0:N

mortlist = c(mLo, mHi)


# setwd("")

## Read in one-patch data for m=0.05
# Data generated from nonspatialPipAndFixation.r
  d = read.table ("nonspatialPipAndFixation.N100.mort0.05.tee9.spatDom10.dat", header=T)

nsmLo1 = max(d$invader)
q1OnePatchmLo = matrix (0, nsmLo1, nsmLo1)
for (i in 1:nsmLo1) q1OnePatchmLo[,i] = d[d$invader==i,]$q1

## Read in 1-patch data for m = 0.5
# Data generated from nonspatialPipAndFixation.r
  d = read.table ("nonspatialPipAndFixation.N100.mort0.5.tee9.spatDom10.dat", header = T)  

nsmHi1 = max(d$invader)
q1OnePatchmHi = matrix (0, nsmHi1, nsmHi1)
for (i in 1:nsmHi1) q1OnePatchmHi[,i] = d[d$invader==i,]$q1

## Read in 2-patch data for m = 0.05
# Data generated from spatialPipAndFixation2Parallel.r
  d = read.table ("spatialPipAndFixation.N100.mort0.05.tee9.spatDom10.dat", header=T)  

nsmLo2 = max(d$invader)
q1TwoPatchmLo = matrix (0, nsmLo2, nsmLo2)
for (i in 1:nsmLo2) q1TwoPatchmLo[,i] = d[d$invader==i,]$q1

## Read in 2-patch data for m = 0.5
# Data generated from spatialPipAndFixation2Parallel.r
  d = read.table ("spatialPipAndFixation.N100.mort0.50.tee9.spatDom10.dat", header = T)  

nsmHi2 = max(d$invader)
q1TwoPatchmHi = matrix (0, nsmHi2, nsmHi2)
for (i in 1:nsmHi2) q1TwoPatchmHi[,i] = d[d$invader==i,]$q1

par (mfrow = c(2,2), oma=c(0,4,4,0), mar=c(5.1,4.1,0.1,2.1))

myGrey = colorRampPalette (c("white", "black"))
lab.cex=1.5; main.cex=1.5

## 1-patch, mLo
sSeries = seq(smin, s0, length=nsmLo1)
image (sSeries, sSeries, q1OnePatchmLo, col=myGrey(100), 
       xlab="Resident offspring size", ylab="Invader offspring size",
       cex.lab=lab.cex, cex.main=main.cex)

## find the size that maximizes expected fitness
tmp_fec_s <- fec(sSeries,mLo,f0)*h
maxExpFitness = sSeries[which(tmp_fec_s==max(tmp_fec_s))]
points (maxExpFitness, maxExpFitness, col="white", pch=8, cex=1.5)

## 1-patch, mHi
sSeries = seq(smin, s0, length=nsmHi1)
image (sSeries, sSeries, q1OnePatchmHi, col=myGrey(100), 
       xlab="Resident offspring size", ylab="Invader offspring size",
       cex.lab=lab.cex, cex.main=main.cex)

tmp_fec_s <- fec(sSeries,mHi,f0)*h
maxExpFitness = sSeries[which(tmp_fec_s==max(tmp_fec_s))]
points (maxExpFitness, maxExpFitness, col="white", pch=8, cex=1.5)

## 2-patch, mLo
sSeries = seq(smin, s0, length=nsmLo2)
image (sSeries, sSeries, q1TwoPatchmLo, col=myGrey(100), 
       xlab="Resident offspring size", ylab="Invader offspring size",
       cex.lab=lab.cex, cex.main=main.cex)

tmp_fec_s <- fec(sSeries,mLo,f0)*h
maxExpFitness = sSeries[which(tmp_fec_s==max(tmp_fec_s))]
points (maxExpFitness, maxExpFitness, col="white", pch=8, cex=1.5)

## 2-patch, mHi
sSeries = seq(smin, s0, length=nsmHi2)
image (sSeries, sSeries, q1TwoPatchmHi, col=myGrey(100), 
       xlab="Resident offspring size", ylab="Invader offspring size",
       cex.lab=lab.cex, cex.main=main.cex)

tmp_fec_s <- fec(sSeries,mHi,f0)*h
maxExpFitness = sSeries[which(tmp_fec_s==max(tmp_fec_s))]
points (maxExpFitness, maxExpFitness, col="white", pch=8, cex=1.5)

mtext ("Two patches", side=2, line=1.5, outer=TRUE, adj=0.2, cex=lab.cex)
mtext ("One patch", side=2, line=1.5, outer=TRUE, adj=0.8, cex=lab.cex)
mtext ("m = 0.05", side=3, line=1.5, outer=TRUE, adj=0.2, cex=lab.cex)
mtext ("m = 0.5", side=3, line=1.5, outer=TRUE, adj=0.8, cex=lab.cex)


# setwd("")
# install.packages("extrafont")
library(extrafont)
loadfonts(device="postscript")
dev.copy2pdf (file="Figure_5.pdf", family="ArialMT",useDingbats=FALSE)
# Embed text for Am Nat
Sys.setenv(R_GSCMD="/usr/local/bin/gs")
embed_fonts ("Figure_5.pdf",outfile='Figure_5.pdf')

