###############################################################################
#
# Code to produce Figure 6 in: Burgess, Snyder, Rountree (xxxx)
# Collective dispersal leads to variance in fitness and maintains
# offspring size variation within marine populations. The American
# Naturalist #: ##-##
#
# IMPORTANT: This code only generates the figure.  To get the required
# data, run nonspatialPipAndFixation.r and
# spatialPipAndFixation2Parallel.r and save the output with
# appropriate file names.
#
# Based on in-house script called makeTwoTraitCoex1Patch2PatchGreyscale2.r
#
##############################################################################


N = 100  			# number of sites
delta = 0.8  		# probability of adult death
s0 = 0.0103			# larval size at which devlopment takes t0 days
smin = 0.001		# minimum viable larval size
t0 = 14         	# pelagic larval duration for initial size s0
mLo = 0.05          # larval mortality rate
mHi = 0.5
tau = 15			# "Eddy time scale" --- 15 days acc. to Siegel et al., 2008
h = 0.5				# probability of hitting the coastline with viable habitat
f0 = 100*s0			# reproductive investment
spatialDomainToEddySize = 10     # spatial domain / eddy size

## PLD as a function of s
t = function (s) {
  (s0/s - 1) + t0
}

## Per capita number of larvae post-mortality
fec = function (s, mort, f0) {
  f0/s * exp(-mort*t(s))
}

# Number of water parcels among which larvae are divided
M = function (tee) {
  ceiling ((tee / tau) * spatialDomainToEddySize)
}

mortlist = c(mLo, mHi)

# setwd("")

## Read in one-patch data for m=0.05
# Data generated from nonspatialPipAndFixation.r
d = read.table ("nonspatialPipAndFixation.N100.mort0.05.tee9.spatDom10.dat", header=T)  

nsmLo1 = max(d$residents)
mLoOnePatchL1 = matrix (0, nsmLo1, nsmLo1); mLoOnePatchL2 = matrix (0, nsmLo1, nsmLo1)
for (i in 1:nsmLo1) {
  mLoOnePatchL1[,i] = d[d$residents==i,]$lifetimeInv1
  mLoOnePatchL2[,i] = d[d$residents==i,]$lifetimeInv2
}
## Find log(expected coexistence time)
mLoOnePatchBoth = matrix (0, nsmLo1, nsmLo1)
for (i in 1:nsmLo1) {
  for (j in 1:nsmLo1) {
    mLoOnePatchBoth[i,j] = log(min(c(mLoOnePatchL1[i,j], mLoOnePatchL2[i,j])))
  }
}

## Read in 1-patch data for m = 0.5
# Data generated from nonspatialPipAndFixation.r
  d = read.table ("nonspatialPipAndFixation.N100.mort0.5.tee9.spatDom10.dat", header = T)  


nsmHi1 = max(d$residents)
mHiOnePatchL1 = matrix (0, nsmHi1, nsmHi1); mHiOnePatchL2 = matrix (0, nsmHi1, nsmHi1)
for (i in 1:nsmHi1) {
  mHiOnePatchL1[,i] = d[d$residents==i,]$lifetimeInv1
  mHiOnePatchL2[,i] = d[d$residents==i,]$lifetimeInv2
}
## Find log(expected coexistence time)
mHiOnePatchBoth = matrix (0, nsmHi1, nsmHi1)
for (i in 1:nsmHi1) {
  for (j in 1:nsmHi1) {
    mHiOnePatchBoth[i,j] = log(min(c(mHiOnePatchL1[i,j], mHiOnePatchL2[i,j])))
  }
}

## Read in 2-patch data for m = 0.05
# Data generated from spatialPipAndFixation2Parallel.r
  d = read.table ("spatialPipAndFixation.N100.mort0.05.tee9.spatDom10.dat", header=T)  

nsmLo2 = max(d$residents)
mLoTwoPatchL1 = matrix (0, nsmLo2, nsmLo2); mLoTwoPatchL2 = matrix (0, nsmLo2, nsmLo2)
for (i in 1:nsmLo2) {
  ## residents column = s2 ==> the first index (row number, y-axis) of
  ## mLoTwoPatchL1, L2 is the invader/s1
  mLoTwoPatchL1[,i] = d[d$residents==i,]$lifetimeInv1
  mLoTwoPatchL2[,i] = d[d$residents==i,]$lifetimeInv2
}
## Find log(expected coexistence time)
mLoTwoPatchBoth = matrix (0, nsmLo2, nsmLo2)
for (i in 1:nsmLo2) {
  for (j in 1:nsmLo2) {
    mLoTwoPatchBoth[i,j] = log(min(c(mLoTwoPatchL1[i,j], mLoTwoPatchL2[i,j])))
  }
}

## Read in 2-patch data for m = 0.5
# Data generated from spatialPipAndFixation2Parallel.r
  d = read.table ("spatialPipAndFixation.N100.mort0.50.tee9.spatDom10.dat", header = T)  

nsmHi2 = max(d$residents)
mHiTwoPatchL1 = matrix (0, nsmHi2, nsmHi2); mHiTwoPatchL2 = matrix (0, nsmHi2, nsmHi2)
for (i in 1:nsmHi2) {
  mHiTwoPatchL1[,i] = d[d$residents==i,]$lifetimeInv1
  mHiTwoPatchL2[,i] = d[d$residents==i,]$lifetimeInv2
}
## Find log(expected coexistence time)
mHiTwoPatchBoth = matrix (0, nsmHi2, nsmHi2)
for (i in 1:nsmHi2) {
  for (j in 1:nsmHi2) {
    mHiTwoPatchBoth[i,j] = log(min(c(mHiTwoPatchL1[i,j], mHiTwoPatchL2[i,j])))
  }
}

#set up layout
temp1 = rbind (matrix(1,6,6), matrix(2,6,6))
temp2 = rbind (matrix(3,6,6), matrix(4,6,6))
layout (mat=cbind(temp1, temp2, matrix(5,12,2)))

lab.cex = 1.5

#Set min and max z values
zmin = min(mLoOnePatchBoth, mHiOnePatchBoth, mLoTwoPatchBoth, mHiTwoPatchBoth)
zmax = max(mLoOnePatchBoth, mHiOnePatchBoth, mLoTwoPatchBoth, mHiTwoPatchBoth)

#Define greyscale
myGrey = colorRampPalette (c("white", "black"))

## mLo ################3

## One patch
## Set up the series of larval sizes
sSeries = seq(smin, s0, length=nsmLo1)
## Find which size maximizes expected fitness.
tmp_fec_s <- fec(sSeries,mLo,f0)*h
maxExpFitness = sSeries[which(tmp_fec_s==max(tmp_fec_s))]

par (mar=c(5,4.5,3,1))
image (sSeries, sSeries, mLoOnePatchBoth,
       xlab="", ylab="",
       col=myGrey(100), xaxt="n", yaxt="n", zlim=c(zmin, zmax))
points (maxExpFitness, maxExpFitness, col="white", pch=8, cex=1.5)
mtext ("Type 2 offspring size", outer=F, line=1, side=1, cex=1.3)
mtext ("Type 1 offspring size", outer=F, line=0.9, side=2, cex=1.3)
mtext ("m = 0.05", side=3, line=0.5, adj=0.5, cex=lab.cex)
mtext ("One patch", side=2, line=2.8, adj=0.5, cex=lab.cex)

## Two patches
sSeries = seq(smin, s0, length=nsmLo2)
image (sSeries, sSeries, mLoTwoPatchBoth,
       xlab="", ylab="",
       col=myGrey(100), xaxt="n", yaxt="n", zlim=c(zmin, zmax))
points (maxExpFitness, maxExpFitness, col="white", pch=8, cex=1.5)
mtext ("Type 2 offspring size", outer=F, line=1, side=1, cex=1.3)
mtext ("Type 1 offspring size", outer=F, line=0.9, side=2, cex=1.3)
mtext ("Two patches", side=2, line=2.8, adj=0.5, cex=lab.cex)


## mHi ##################3

## One patch
sSeries = seq(smin, s0, length=nsmHi1)
tmp_fec_s <- fec(sSeries,mHi,f0)*h
maxExpFitness = sSeries[which(tmp_fec_s==max(tmp_fec_s))]

image (sSeries, sSeries, mHiOnePatchBoth, 
       xlab="", ylab="",
       col=myGrey(100), xaxt="n", yaxt="n", zlim=c(zmin, zmax))
points (maxExpFitness, maxExpFitness, col="white", pch=8, cex=1.5)
mtext ("Type 2 offspring size", outer=F, line=1, side=1, cex=1.3)
mtext ("Type 1 offspring size", outer=F, line=0.9, side=2, cex=1.3)
mtext ("m = 0.5", side=3, line=0.5, outer=F, adj=0.5, cex=lab.cex)

## Two patches
sSeries = seq(smin, s0, length=nsmHi2)
image (sSeries, sSeries, mHiTwoPatchBoth,
       xlab="", ylab="",
       col=myGrey(100), xaxt="n", yaxt="n", zlim=c(zmin, zmax))
points (maxExpFitness, maxExpFitness, col="white", pch=8, cex=1.5)
mtext ("Type 2 offspring size", outer=F, line=1, side=1, cex=1.3)
mtext ("Type 1 offspring size", outer=F, line=0.9, side=2, cex=1.3)


l.y=seq(zmin, zmax, length=100);
image(list(x=1:2,y=l.y,z=rbind(l.y,l.y)),col=myGrey(100), bty="o",xaxt="n",yaxt="n");
tickLabels = c(1, 10, 10^2, 10^3, 10^4, 10^5, 10^6, 10^7)
tickLabels = tickLabels[tickLabels < exp(zmax)]
tickLocations = log(tickLabels)
##tickLocations = seq(zmin, zmax, length=10)
##tickLabels = round(exp(tickLocations))
axis(side=2,cex.axis=1.1,at=tickLocations, label=tickLabels, cex.axis=1.3)
mtext("Coexistence time (years)",side=2,line=3,cex=1.4)

# setwd("")
# install.packages("extrafont")
library(extrafont)
loadfonts(device="postscript")
dev.copy2pdf (file="Figure_6.pdf", family="ArialMT",useDingbats=FALSE)
# Embed text for Am Nat
Sys.setenv(R_GSCMD="/usr/local/bin/gs")
embed_fonts ("Figure_6.pdf",outfile='Figure_6.pdf')

