###############################################################################
#
# Code to produce Figure A1 in Appendix A of: Burgess, Snyder,
# Rountree (xxxx) Collective dispersal leads to variance in fitness
# and maintains offspring size variation within marine
# populations. The American Naturalist #: ##-##
#
###############################################################################

ns1 = 6 			# number of sizes s to use
ntee1 = 15 			# number of spawning durations T to use 
N = 100  			# number of sites
delta = 0.8  		# probability of adult death
s0 = 0.0103			# larval size at which devlopment takes t0 days
smin = 0.001		# minimum viable larval size
t0 = 14         	# pelagic larval duration for initial size s0
mLo = 0.05          # larval mortality rate
mHi = 0.5
mort = mLo
tau = 15			# "Eddy time scale" --- 15 days acc. to Siegel et al., 2008
h = 0.5				# probability of hitting the coastline with viable habitat
f0 = 100*s0			# reproductive investment
spatialDomainToEddySize = 10     # spatial domain / eddy size

n1 = 1
n2 = N - 1

# PLD as a function of s
t = function (s) {
  (s0/s - 1) + t0
}

# Per capita number of larvae post-mortality
fec = function (s, mort, f0) {
  f0/s * exp(-mort*t(s))
}

# Number of water parcels among which larvae are divided
M = function (tee) {
  ceiling ((tee / tau) * spatialDomainToEddySize)
}

# Define types
n1Series = 0:N
s1Series = seq(smin, s0, length=ns1)		# vector of larval sizes
tee1Series = seq (9, 180, length=ntee1)   	# vector of spawning durations
tee2 = 100

## Type 2 has the size which maximizes expected fitness
tmp_fec_s <- fec(s1Series,mort,f0)*h
s2 = s1Series[which(tmp_fec_s==max(tmp_fec_s))]

numreps = 100000
simMeanVarLambda = function (s1, s2, tee1, tee2) {
  w1 = numeric (numreps)
  for (i in 1:numreps) {
    ## y1 = number of type 1 settlers that reach habitat
    y1 = fec(s1,mort,f0)*n1/M(tee1) * rbinom (1, prob=h, size=M(tee1))
    y2 = fec(s2,mort,f0)*n2/M(tee2) * rbinom (1, prob=h, size=M(tee2))
    ## w1 = number of type 1 recruits + type 1 survivors (= fitness)
    if (y1 + y2 == 0) {
      w1[i] = (1 - delta)*n1
    } else {
      w1[i] = y1/(y1 + y2) * delta*N + (1 - delta)*n1
    }
  }
  meanw = mean(w1); varw = var(w1)
  geomMean = exp(mean(log(w1)))
  return (list(mean=meanw, var=varw, geom=geomMean))
}

getMeanTimeToFixation = function (s1, s2, tee1, tee2) {
  b1 = fec(s1, mort, f0); b2 = fec(s2, mort, f0)
  m1 = M (tee1); m2 = M (tee2)
  P = matrix (0, N+1, N+1)
  
  ## Set up transition matrix #############
  ## loop over current value of p
  for (n1 in n1Series) {
    p = n1/N
    index = n1 + 1
    ## What are the possible transitions we could make from current
    ## value of p to pNew?

    ## loop over number of successful type 1 packets
    for (k1 in 0:m1) {
      ## loop over number of successful type 2 packets
      for (k2 in 0:m2) {
        ## Calculate the updated value of p.
        numType1Larvae = b1*k1*p/m1
        numType2Larvae = b2*k2*(1 - p)/m2
        pNew = delta*numType1Larvae / (numType1Larvae  + numType2Larvae) +
          (1 - delta)*p
        if ((numType1Larvae == 0) & (numType2Larvae == 0)) pNew = p
        
        ## Round the updated n to the nearest integer since the
        ## proportion of space won is deterministic.
        n1New = round(pNew*N)
        pNew = n1New/N
        newIndex = n1New + 1
        
        ## Calculate the probability of getting k1 packets for type
        ## 1 and k2 packets for type 2.
        prob = dbinom (k1, m1, h) * dbinom (k2, m2, h)

        ## Fill in the relevant column of the transition matrix. 
        P[newIndex, index] = P[newIndex, index] + prob
      }
    }
  }

  ## Calculate the fundamental matrix of the transient states
  fundN = solve (diag(N-1) - P[2:N, 2:N])

  ## Mean time with both types if type 1 is invading
  tmp = apply (fundN, 2, sum)
  lifetimeInv1 = tmp[1]
  return (lifetimeInv1)
}

meanFitness = matrix (NA, ns1, ntee1)
varFitness = matrix (NA, ns1, ntee1)
meanTimeToFixation = matrix (NA, ns1, ntee1)

for (i in 1:ns1) {
  s1 = s1Series[i]
  for (j in 1:ntee1) {
    tee1 = tee1Series[j]
    out = simMeanVarLambda (s1, s2, tee1, tee2)
    meanFitness[i,j] = out$mean
    varFitness[i,j] = out$var
    meanTimeToFixation[i,j] = getMeanTimeToFixation (s1, s2, tee1, tee2)
  }
}

# Make plot for paper
quartz(width=5,height=5)
par (cex.lab=1.4, oma=c(0,2,1,1))
ymin = min (meanTimeToFixation)
ymax = (max (meanTimeToFixation))
xmin = min (varFitness)
xmax = max (varFitness)
cols=c("red", "orange", "black", "green", "blue", "violet")
pchs=0:5
plot (varFitness[1,], meanTimeToFixation[1,], log = "y",
      xlim=c(xmin, xmax), ylim=c(ymin, ymax),
      xlab="Fitness variance", ylab="", col=cols[1],las=1,pch=pchs[1])
mtext("Mean time (years) to fixation",side=2,line=3.5,cex=1.4)
points (varFitness[2,], meanTimeToFixation[2,], col=cols[2],pch=pchs[2])
points (varFitness[3,], meanTimeToFixation[3,], col=cols[3],pch=pchs[3])
points (varFitness[4,], meanTimeToFixation[4,], col=cols[4],pch=pchs[4])
points (varFitness[5,], meanTimeToFixation[5,], col=cols[5],pch=pchs[5])
points (varFitness[6,], meanTimeToFixation[6,], col=cols[6],pch=pchs[6])
legend (x="topright",
        legend=c(paste("s1 = ", s1Series[1], sep=""),
            paste("s1 = ", s1Series[2], sep=""),
            paste("s1 = ", s1Series[3], sep=""),
            paste("s1 = ", s1Series[4], sep=""),
            paste("s1 = ", s1Series[5], sep=""),
            paste("s1 = ", s1Series[6], sep="")),
        col=cols,
        pch=pchs)
        


# setwd("")
# install.packages("extrafont")
library(extrafont)
loadfonts(device="postscript")
dev.copy2pdf (file="Figure_A1.pdf", family="ArialMT",useDingbats=FALSE)
# Embed text for Am Nat
Sys.setenv(R_GSCMD="/usr/local/bin/gs")
embed_fonts ("Figure_A1.pdf",outfile='Figure_A1.pdf')

# Save data in the workspace
save.image(file="Figure_A1.RData")