###########################################################################
#
# nonspatialPipAndFixation.r: generates both fixation probability and
# expected lifetimes for the one-patch/nonspatial model and outputs to
# file.  No graphics.
#
###########################################################################

ns = 10
N = 100 # number of sites
delta = 0.8  # prob. of adult death

s0 = 0.0103			# larval size at which devlopment takes t0 days
smin = 0.001		# Minimum viable larval size
t0 = 14         	# PLD for initial size s0
mLo = 0.05                      # Larval mortality rate
mHi = 0.5
tau = 15			# "Eddy time scale" --- 15 days acc. to Siegel et al., 2008
h = 0.5				# percent of coastline that is viable habitat
f0 = 100*s0
spatialDomainToEddySize = 10     # Spatial domain / eddy size

mort = mHi
tee1 = 9; tee2 = tee1

n1 = 1
n2 = N - 1

# PLD as a function of s
t = function (s) {
  (s0/s - 1) + t0
}

# Per capita number of larvae post-mortality
fec = function (s, mort, f0) {
  f0/s * exp(-mort*t(s))
}

# Number of water parcels among which larvae are divided
M = function (tee) {
  ceiling ((tee / tau) * spatialDomainToEddySize)
}

n1Series = 0:N
sSeries = seq(smin, s0, length=ns)	# vector of larval sizes

getResults = function (s1, s2, tee1, tee2) {
  b1 = fec(s1, mort, f0); b2 = fec(s2, mort, f0)
  m1 = M (tee1); m2 = M (tee2)
  P = matrix (0, N+1, N+1)
  
  ## Set up transition matrix #############
  ## loop over current value of p
  for (n1 in n1Series) {
    p = n1/N
    index = n1 + 1
    ## What are the possible transitions we could make from current
    ## value of p to pNew?
    
    ## loop over number of successful type 1 packets
    for (k1 in 0:m1) {
      ## loop over number of successful type 2 packets
      for (k2 in 0:m2) {
        ## Calculate the updated value of p.
        numType1Larvae = b1*k1*p/m1
        numType2Larvae = b2*k2*(1 - p)/m2
        pNew = delta*numType1Larvae / (numType1Larvae  + numType2Larvae) +
          (1 - delta)*p
        if ((numType1Larvae == 0) & (numType2Larvae == 0)) pNew = p
        
        ## Round the updated n to the nearest integer since the
        ## proportion of space won is deterministic.
        n1New = round(pNew*N)
        pNew = n1New/N
        newIndex = n1New + 1
        
        ## Calculate the probability of getting k1 packets for type
        ## 1 and k2 packets for type 2.
        prob = dbinom (k1, m1, h) * dbinom (k2, m2, h)

        ## Fill in the relevant column of the transition matrix. 
        P[newIndex, index] = P[newIndex, index] + prob
      }
    }
  }

  ## a1 is the probability of fixating at p = 1 in the next time step.
  a1 = P[N + 1, 2:N]

  ## Calculate the fundamental matrix of the transient states
  Q = P[2:N, 2:N]
  fundN = solve (diag(N - 1) - Q)

  q1vec = a1 %*% fundN
  ## Start with 1 individ. of type 1 
  q1 = q1vec[1]
  
  ## Mean time with both types if type 1 is invading
  tmp = apply (fundN, 2, sum)
  lifetimeInv1 = tmp[1]
  lifetimeInv2 = tmp[N-1]

  out = list (q1=q1, lifetimeInv1=lifetimeInv1, lifetimeInv2=lifetimeInv2)
  
  return (out)
}

lifetime1 = matrix (NA, ns, ns)
lifetime2 = matrix (NA, ns, ns)
both = matrix (NA, ns, ns)

cat ("N = ", N, "mort = ", mort, "tee = ", tee1, "\n")
filename = paste ("nonspatialPipAndFixation.N", N, ".mort", mort, ".tee", tee1, ".spatDom", spatialDomainToEddySize, ".dat", sep="")
d = NULL
write.table ("residents invader q1 lifetimeInv1 lifetimeInv2",
             filename, row.names=FALSE, col.names=FALSE)

for (i in 1:ns) { # resident
  for (j in 1:ns) { # invader
    cat (i, j, "\n")
    s1 = sSeries[j]; s2 = sSeries[i]
    out = getResults (s1, s2, tee1, tee2)
    
    d = data.frame(residents=i, invader=j, q1=out$q1,
        lifetimeInv1=out$lifetimeInv1, lifetimeInv2=out$lifetimeInv2)
    write.table (d, file=filename, append=TRUE,
                 row.names=FALSE, col.names=FALSE)
  }
}
