############################################################################
#
# spatialPipAndFixation2.r: generates two-patch PIP and fixation plots.
# Like MarkovMV2.r, this code tracks which packets came from which
# patch from the beginning, so that no more than m packets can come
# from one patch.
#
# Notes for the parallel version:
# This is the parallel version, which might be invoked as follows:
#
# 	srun --ntasks=2601 --nodes=73 --ntasks-per-node=36 	\
#		${RPATH}/Rscript 				\
#		${SCRIPTPATH}/spatialPipAndFixation2Parallel.r 	\
#		100 0.50 1
#	
# The first parameter is N, the number of sites.
# The second parameter is mort, the larval mortalilty rate.
# The third parameter is the outer loop index, as the code was parallelized
#	to allow the outer loops to run independently on multiple processor
#	cores.
#
# The above example relies on the srun command found in the SLURM/MOAB 
# parallel computing environment.  MPICH and OpenMPI will have different
# methods to launch parallel jobs.  The only dependence that might need
# to be changed is the PMI_RANK environment variable; replace with whatever
# variable your system uses to track MPI ranks.
#
# There is one FIXME that we didn't get around to.  IIRC, every rank 
# will execute the following line of code:
#
#    write.table ("residents invader q1 lifetimeInv1 lifetimeInv2", \
# 	filename, row.names=FALSE, col.names=FALSE)
#
# The result is that this file is written and overwritten by each rank.
# Subsequent writes to this file will only be performed by rank 0, and 
# there is sufficent synchronization that the last superfluous re-write is 
# completed well before data is written to the file.  Fixing this should
# should just be a matter of moving this line into the following
#
# 	if( mpi_rank == 0 ) 
#
# clause, but I am reluctant to do so without making resources available to
# test the patch.  As it stands, this is a venial bug, not a mortal one.
#
# Questions and complaints about the code parallelization should be 
# directed to Barry Rountree (rountree@llnl.gov).  
#
############################################################################

driver <- function(){
  USING_MPI <- ( "" != Sys.getenv("PMI_RANK", names=FALSE) )

  if( USING_MPI ){				# https://cran.r-project.org/web/packages/Rmpi/index.html
    library("Rmpi") 			# http://www.stats.uwo.ca/faculty/yu/Rmpi/
    mpi_rank <- mpi.comm.rank(comm=0)
    mpi_size <- mpi.comm.size(comm=0)
  }else{
    stop("Environment variable PMI_RANK not detected, halting.\n")
  }

  numPatch = 2  # number of patches.  This code only works for
                # numpatch = 2 so far.
  patchProb = 1/numPatch   # probability of landing in a given patch
  ns = 10                  # number of larval sizes to loop over
  delta = 0.8  			# prob. of adult death

  s0 = 0.0103			# larval size at which devlopment takes t0 days
  smin = 0.001			# Minimum viable larval size
  t0 = 14         		# PLD for initial size s0
  tau = 15			# "Eddy time scale" --- 15 days acc. to Siegel et al., 2008
  h = 0.5				# percent of coastline that is viable habitat
  f0 = 100*s0
  spatialDomainToEddySize = 10     # Spatial domain / eddy size

  tee1 = 9; tee2 = tee1
  sSeries = seq(smin, s0, length=ns)	# vector of larval sizes
  q1s = matrix (NA, ns, ns)

  ## PLD as a function of s
  t <- function (s) {
    (s0/s - 1) + t0
  }

  ## Per capita number of larvae post-mortality
  fec <- function (s, mort, f0) {
    f0/s * exp(-mort*t(s))
  }

  ## Number of water parcels among which larvae are divided
  M = function (tee) {
    ceiling ((tee / tau) * spatialDomainToEddySize)
  }


  ## outputs all possible configurations of total objects distributed
  ## among 2 bins 

  allConfig2 = function (maxPackets) {
    ## return value is 
    ## c[num. from source patch to patch 1,
    ##   num. from source patch to patch 2],
    ##   prob. of that configuration]
    
    numConfig = (maxPackets+1)*(maxPackets+2)/2
    ## config[i,] =
    ##    c(num. landing in patch 1, num. landing in patch 2, total landing)
    config = matrix (NA, numConfig, 3)
    index = 1
    for (i in 0:maxPackets) { # returned to land out of total
      for (j in 0:i) { # of those landing in patch 1, how many from patch 1
        config[index,] = c(j, i-j, i)
        index = index + 1
      }
    }
    return (data.frame(to1=config[,1], to2=config[,2],
                       prob=dbinom(config[,3], maxPackets, h) *
                         dbinom (config[,1], config[,3], 0.5))
            )
  }

  parallel_getP <- function (s1, s2, tee1, tee2, ns_i, ns_j) {
    m1 = M (tee1); m2 = M (tee2)
    b1 = fec(s1, mort, f0); b2 = fec(s2, mort, f0)

    ## Transition matrix for state cross-classified by the frequency
    ## of type 1 in patch 1 and patch 2.  Patch 1 is slowest variable,
    ## patch 2 is second slowest, etc.  Thus, for a 2-patch system,
    ## (0, 0) (no type 1 in patch 1 or 2) is state 1, (0, 1/NPatch)
    ## (no type 1 in patch 1, one site occupied by type 1 in patch 2)
    ## is state 2, (0, NPatch) is state NPatch+1, (1, 0) is state
    ## NPatch+2, etc.

    numStates = (NPatch+1)^numPatch
    P = matrix (0, numStates, numStates)

#################################################################
    ## Create the configurations we'll need.
##################################################################
    
    ## full config: type 1's and 2's in both patches 
    ## configurations of types 1 and 2 from patches 1 and 2
    fromPatch1Type1 = fromPatch2Type1 = allConfig2 (m1)
    fromPatch1Type2 = fromPatch2Type2 = allConfig2 (m2)

    ## type 1
    numConfigFromPatch1 = length (fromPatch1Type1$prob)
    numConfigFromPatch2 = length (fromPatch2Type1$prob)
    t1f1 = t2f1 = t1f2 = t2f2 = prob = 
      numeric (numConfigFromPatch1*numConfigFromPatch2)
    index = 0
    ## loop through all configurations of arriving type 1 packets from patch
    ## 1 and patch 2     
    for (i in 1:numConfigFromPatch1) {
      for (j in 1:numConfigFromPatch2) {
        index = index + 1
        ## packets coming from patch 1 to patch 1
        t1f1[index] = fromPatch1Type1$to1[i]
        ## packets coming from patch 1 to patch 2
        t2f1[index] = fromPatch1Type1$to2[i]
        ## packets coming from patch 2 to patch 1
        t1f2[index] = fromPatch2Type1$to1[j]
        ## packets coming from patch 2 to patch 2
        t2f2[index] = fromPatch2Type1$to2[j]
        ## probability that this joint configuration occurred
        prob[index] = fromPatch1Type1$prob[i] * fromPatch2Type1$prob[j]
      }
    }
    c1Normal = data.frame (t1f1=t1f1, t1f2=t1f2, t2f1=t2f1, t2f2=t2f2, prob=prob)

    ## type 2
    numConfigFromPatch1 = length (fromPatch1Type2$prob)
    numConfigFromPatch2 = length (fromPatch2Type2$prob)
    t1f1 = t2f1 = t1f2 = t2f2 = prob = numeric (numConfigFromPatch1*numConfigFromPatch2)
    index = 0
    ## loop through all configurations of arriving type 2 packets from patch
    ## 1 and patch 2 
    for (i in 1:numConfigFromPatch1) {
      for (j in 1:numConfigFromPatch2) {
        index = index + 1
        t1f1[index] = fromPatch1Type2$to1[i]
        t2f1[index] = fromPatch1Type2$to2[i]
        t1f2[index] = fromPatch2Type2$to1[j]
        t2f2[index] = fromPatch2Type2$to2[j]
        prob[index] = fromPatch1Type2$prob[i] * fromPatch2Type2$prob[j]
      }
    }
    c2Normal = data.frame (t1f1=t1f1, t1f2=t1f2, t2f1=t2f1, t2f2=t2f2, prob=prob)

    ## if there are no type 1's in patch 1
    fromPatch2Type1 = allConfig2 (m1)
    fromPatch1Type2 = fromPatch2Type2 = allConfig2 (m2)

    fromPatch1Type1 = data.frame(to1=0, to2=0, prob=1)
    numConfigFromPatch1 = 1
    numConfigFromPatch2 = length (fromPatch2Type1$prob)
    t1f1 = t2f1 = t1f2 = t2f2 = prob = numeric (numConfigFromPatch1*numConfigFromPatch2)
    index = 0
    for (i in 1:numConfigFromPatch1) {
      for (j in 1:numConfigFromPatch2) {
        index = index + 1
        t1f1[index] = fromPatch1Type1$to1[i]
        t2f1[index] = fromPatch1Type1$to2[i]
        t1f2[index] = fromPatch2Type1$to1[j]
        t2f2[index] = fromPatch2Type1$to2[j]
        prob[index] = fromPatch1Type1$prob[i] * fromPatch2Type1$prob[j]
      }
    }
    c1No1Patch1 = data.frame (t1f1=t1f1, t1f2=t1f2, t2f1=t2f1, t2f2=t2f2, prob=prob)

    ## if there are no type 1's in patch 2
    fromPatch1Type1 = allConfig2 (m1)
    fromPatch1Type2 = fromPatch2Type2 = allConfig2 (m2)

    numConfigFromPatch1 = length (fromPatch1Type1$prob)
    fromPatch2Type1 = data.frame(to1=0, to2=0, prob=1)
    numConfigFromPatch2 = 1
    t1f1 = t2f1 = t1f2 = t2f2 = prob = numeric (numConfigFromPatch1*numConfigFromPatch2)
    index = 0
    for (i in 1:numConfigFromPatch1) {
      for (j in 1:numConfigFromPatch2) {
        index = index + 1
        t1f1[index] = fromPatch1Type1$to1[i]
        t2f1[index] = fromPatch1Type1$to2[i]
        t1f2[index] = fromPatch2Type1$to1[j]
        t2f2[index] = fromPatch2Type1$to2[j]
        prob[index] = fromPatch1Type1$prob[i] * fromPatch2Type1$prob[j]
      }
    }
    c1No1Patch2 = data.frame (t1f1=t1f1, t1f2=t1f2, t2f1=t2f1, t2f2=t2f2, prob=prob)

    ## if there are no type 2's in patch 1
    fromPatch1Type1 = fromPatch2Type1 = allConfig2 (m1)
    fromPatch2Type2 = allConfig2 (m2)
    
    fromPatch1Type2 = data.frame(to1=0, to2=0, prob=1)
    numConfigFromPatch1 = 1
    numConfigFromPatch2 = length (fromPatch2Type2$prob)
    t1f1 = t2f1 = t1f2 = t2f2 = prob = numeric (numConfigFromPatch1*numConfigFromPatch2)
    index = 0
    for (i in 1:numConfigFromPatch1) {
      for (j in 1:numConfigFromPatch2) {
        index = index + 1
        t1f1[index] = fromPatch1Type2$to1[i]
        t2f1[index] = fromPatch1Type2$to2[i]
        t1f2[index] = fromPatch2Type2$to1[j]
        t2f2[index] = fromPatch2Type2$to2[j]
        prob[index] = fromPatch1Type2$prob[i] * fromPatch2Type2$prob[j]
      }
    }
    c2No2Patch1 = data.frame (t1f1=t1f1, t1f2=t1f2, t2f1=t2f1, t2f2=t2f2, prob=prob)

    ## if there are no type 2's in patch 2
    fromPatch1Type1 = fromPatch2Type1 = allConfig2 (m1)
    fromPatch1Type2 = allConfig2 (m2)
    
    numConfigFromPatch1 = length (fromPatch1Type2$prob)
    fromPatch2Type2 = data.frame(to1=0, to2=0, prob=1)
    numConfigFromPatch2 = 1
    t1f1 = t2f1 = t1f2 = t2f2 = prob = numeric (numConfigFromPatch2*numConfigFromPatch2)
    index = 0
    for (i in 1:numConfigFromPatch1) {
      for (j in 1:numConfigFromPatch2) {
        index = index + 1
        t1f1[index] = fromPatch1Type2$to1[i]
        t2f1[index] = fromPatch1Type2$to2[i]
        t1f2[index] = fromPatch2Type2$to1[j]
        t2f2[index] = fromPatch2Type2$to2[j]
        prob[index] = fromPatch1Type2$prob[i] * fromPatch2Type2$prob[j]
      }
    }
    c2No2Patch2 = data.frame (t1f1=t1f1, t1f2=t1f2, t2f1=t2f1, t2f2=t2f2, prob=prob)

    ## if there are no type 1's in either patch
    c1No1Patch1Patch2 = data.frame (t1f1=0, t1f2=0, t2f1=0, t2f2=0, prob=1)

    ## if there are no type 2's in either patch
    c2No2Patch1Patch2 = data.frame (t1f1=0, t1f2=0, t2f1=0, t2f2=0, prob=1)

    ## We have N^2=10,201 processes executing at this point.
    ##
    ## > length(unique( ( 0:((101^2)-1) ) %/% 101 ) ) 
    ## [1] 101
    ## 
    ## Thus, np1 = mpi_rank %/% 101.
    ##
    ## > length(unique( ( 0:((101^2)-1) ) %% 101 ) )
    ## [1] 101
    ##
    ## Thus, np2 = mpi_rank %% 101.

    ## np1 = number of type 1 individuals in patch 1
    ## np2 = number of type 1 individuals in patch 2
    np1 <- mpi_rank %/% (NPatch+1)
    np2 <- mpi_rank %%  (NPatch+1)

    ## pvec = (frequency of type 1 in patch 1, frequency of type 1 in
    ## patch 2)    
    pvec = c(np1, np2)/NPatch
    ## index = the row or column number of this state in the
    ## transition matrix
    index = np1*(NPatch+1) + np2 + 1

    ## determine appropriate appropriate ensemble of configurations
    ## based on the occupancies of patch 1 and patch 2
    
    ## no type 1 in patch 1 or 2
    if ((np1 == 0) & (np2 == 0)) {
      c1 = c1No1Patch1Patch2
      ## no type 1 in patch 1 alone
    } else if (np1 == 0) {
      c1 = c1No1Patch1
      ## no type 1 in patch 2 alone
    } else if (np2 == 0) {
      c1 = c1No1Patch2
      ## type 1 in both patches
    } else {
      c1 = c1Normal
    }

    ## no type 2 in patch 1 or 2
    if ((np1 == NPatch) & (np2 == NPatch)) {
      c2 = c2No2Patch1Patch2
      ## no type 2 in patch 1 alone
    } else if (np1 == NPatch) {
      c2 = c2No2Patch1
      ## no type 2 in patch 2 alone
    } else if (np2 == NPatch) {
      c2 = c2No2Patch2
      ## type 2 in both patches
    } else {
      c2 = c2Normal
    }
    
    numconfig1 = length( c1[,1] )
    numconfig2 = length( c2[,1] )

    pvecNew = rep(0, numPatch)
    ## Calculate the number of type 1 settlers arriving in patch 1
    numType1LarvaePatch1 = as.matrix(c1[,1:2]) %*% (b1/m1*pvec)
    ## Calculate the number of type 1 settlers arriving in patch 2
    numType1LarvaePatch2 = as.matrix(c1[,3:4]) %*% (b1/m1*pvec)
    ## Calculate the number of type 2 settlers arriving in patch 1
    numType2LarvaePatch1 = as.matrix(c2[,1:2]) %*% (b2/m2*(1 - pvec))
    ## Calculate the number of type 2 settlers arriving in patch 2
    numType2LarvaePatch2 = as.matrix(c2[,3:4]) %*% (b2/m2*(1 - pvec))

    ## Do this to get the fancy sapply to work.
    expandedNumType2LarvaePatch1 = rep(numType2LarvaePatch1, numconfig1)
    expandedNumType2LarvaePatch2 = rep(numType2LarvaePatch2, numconfig1)

    ## Do this to get the fancy sapply to work.
    expandedNumType1LarvaePatch1 = as.vector( sapply( numType1LarvaePatch1, function(x) rep(x, numconfig2) ) )
    expandedNumType1LarvaePatch2 = as.vector( sapply( numType1LarvaePatch2, function(x) rep(x, numconfig2) ) )

    ## Calculate the new frequency of type 1 in patch 1
    pvecNew1 = (1 - delta)*pvec[1] + 
      delta*expandedNumType1LarvaePatch1 / (expandedNumType1LarvaePatch1 + expandedNumType2LarvaePatch1)
    ## Calculate the new frequency of type 1 in patch 2
    pvecNew2 = (1 - delta)*pvec[2] + 
      delta*expandedNumType1LarvaePatch2 / (expandedNumType1LarvaePatch2 + expandedNumType2LarvaePatch2)

    ## We don't allow empty sites, so if no larvae arrive, keep pvec the same.
    pvecNew1[ ( expandedNumType1LarvaePatch1 == 0 ) & ( expandedNumType2LarvaePatch1 == 0 ) ] = pvec[1]
    pvecNew2[ ( expandedNumType1LarvaePatch2 == 0 ) & ( expandedNumType2LarvaePatch2 == 0 ) ] = pvec[2]

    ## Convert frequencies to actual numbers
    nvecNew1 = round( pvecNew1 * NPatch )
    nvecNew2 = round( pvecNew2 * NPatch )

    ## Calculate the index of the new cross-classified state.
    newIndex = nvecNew1*(NPatch+1) + nvecNew2 + 1

    ## Also needed for the fancy sapply
    expandedConfigProb2 <<- rep(c2$prob, numconfig1)
    expandedConfigProb1 <<- as.vector( sapply( c1$prob, function(x) rep(x, numconfig2) ) )
    expandedProbk2 <<- rep(c2$probk, numconfig1)
    expandedProbk1 <<- as.vector( sapply( c1$probk, function(x) rep(x, numconfig2) ) )

    ## The total probability for this transition
    prob = expandedConfigProb1 * expandedConfigProb2

    ## The fancy sapply!  Behold its glory!  Or... this used to be an
    ## sapply, says RES.  Now that it's parallel code, it's a loop.
    for( x in unique(newIndex) ){
      P[ x, index ] <- P[ x, index ] + sum( prob[ newIndex==x ] )
    }	

    ##
    ## MPI
    ##
    send_buf <- as.numeric( P[ , index ] )
    recv_buf <- double( numStates^2)
    nevermind <- mpi.gather( x=send_buf, type=2, rdata=recv_buf, root=0, comm=0 )

    if( mpi_rank == 0 ){
      P <- matrix( recv_buf, nrow=numStates, ncol=numStates )
      return (P)
    }else{
      return (NA)
    }
  }


  parallel_driver_2<-function(){

    filename = paste ("spatialPipAndFixation.N", N, ".mort", mort, ".tee", tee1, ".spatDom", spatialDomainToEddySize, ".outerLoopIdx", outerLoopIdx, ".dat", sep="")

    ts_start <- proc.time()
                                        #FIXME only do in rank 0
    write.table ("residents invader q1 lifetimeInv1 lifetimeInv2", filename, row.names=FALSE, col.names=FALSE)
    i <- outerLoopIdx # Resident; this loop is now parallelized.
    for (j in 1:ns) { # invader
      s1 = sSeries[j]; s2 = sSeries[i]
      P = parallel_getP(s1, s2, tee1, tee2, i, j) 
      if( mpi_rank == 0 ){

        ## a1 is the probability of fixating at p = 1 in the next time step.
        a1 = P[numStates, 2:(numStates-1)]
        
        ## Calculate the fundamental matrix of the transient states
        Q = P[2:(numStates-1), 2:(numStates-1)]
        fundN = solve (diag(numStates - 2) - Q)

        q1vec = a1 %*% fundN
        ## Start with 1 individ. of type 1 in each patch
        q1 = q1vec[1]

        ## Now calculate mean time to fixation.  
        ## 
        tmp = apply (fundN, 2, sum)
        ## Mean time with both types if type 1 is invading
        lifetimeInv1 = tmp[1]
        ## Mean time with both types if type 2 is invading
        lifetimeInv2 = tmp[numStates-2]
        out = list (q1=q1, lifetimeInv1=lifetimeInv1, lifetimeInv2=lifetimeInv2)
        d = data.frame(residents=i, invader=j, q1=out$q1, 
                       lifetimeInv1=out$lifetimeInv1, lifetimeInv2=out$lifetimeInv2)
        write.table (d, file=filename, append=TRUE, 
                     row.names=FALSE, col.names=FALSE)
        ts_stop <- proc.time()
        cat("i=", i, "j=", j, "elapsed=", ts_stop-ts_start, "\n"); flush.console()
      }
    }
  }


  parallel_driver_2()
  mpi.barrier( comm = 0 )
  nevermind<-mpi.finalize()

}

options(show.error.locations = TRUE)    
args <- commandArgs( trailingOnly=TRUE )

N 		<- as.integer( args[1] )
mort		<- as.double( args[2] ) 
outerLoopIdx 	<- as.integer( args[3] )
NPatch		<- N/2

driver()

